package main

import (
	"errors"
	"flow"
	"flow/cmd/demo1/devops"
	"flow/flowserver"
	"flow/registry"
	"fmt"
	"log"
	"math"
	"math/rand"
	"net/http"
	"strings"
	"time"

	"github.com/gohxs/prettylog"
	"github.com/gohxs/webu"
	"github.com/gohxs/webu/chain"
)

func main() {
	prettylog.Global()
	log.Println("Running version:", flowserver.Version)

	// String functions
	registry.Describer(
		registry.Add(strings.Split).Inputs("string", "separator"),
		registry.Add(strings.Join).Inputs("", "sep"),
		registry.Add(strings.Compare, strings.Contains),
		registry.Register("Cat", func(a, b string) string { return a + " " + b }),
		registry.Register("ToString", func(a interface{}) string { return fmt.Sprint(a) }),
	).Tags("string").Extra("style", registry.M{"color": "#839"})

	// Math functions
	registry.Add(
		math.Abs, math.Cos, math.Sin, math.Exp, math.Exp2, math.Tanh, math.Max, math.Min,
	).Tags("math").Extra("style", registry.M{"color": "#386"})

	registry.Describer(
		registry.Add(rand.Int, rand.Intn, rand.Float64),
		registry.Register("Perm", func(n int) []int {
			if n > 10 { // Limiter for safety
				n = 10
			}
			return rand.Perm(n)
		}),
	).Tags("rand").Extra("style", registry.M{"color": "#486"})

	// Test functions
	registry.Add(testErrorPanic, testErrorDelayed, testRandomError).
		Tags("testing-errors")

	registry.Describer(
		registry.Register("wait", wait),
		registry.Register("waitRandom", waitRandom),
	).Tags("testing-time").Extra("style", map[string]string{"color": "#8a5"})

	addr := ":2015"
	log.Println("Starting server  at:", addr)

	c := chain.New(webu.ChainLogger(prettylog.New("req")))

	mux := http.NewServeMux()

	mux.Handle("/", c.Build(http.RedirectHandler("/default/", 302).ServeHTTP))
	mux.Handle("/default/", c.Build(flowserver.New(registry.Global, "default").ServeHTTP))

	mux.Handle("/devops/", c.Build(flowserver.New(devops.New(), "devops").ServeHTTP))

	// Context registry
	http.ListenAndServe(addr, mux)
}

func wait(data flow.Data, n int) flow.Data {
	time.Sleep(time.Duration(n) * time.Second) // Simulate
	return data
}
func waitRandom(data flow.Data) flow.Data {
	time.Sleep(time.Duration(rand.Intn(10)) * time.Second)
	return data
}

////////////////////
// Testing custom struct operations
/////

////////////////////////
// testOps
////////////////
func testErrorPanic(n int) flow.Data {
	dur := time.Duration(n)
	time.Sleep(dur * time.Second)
	panic("I panicked")
}

func testErrorDelayed(n int) (flow.Data, error) {
	dur := time.Duration(n)
	time.Sleep(dur * time.Second)
	return nil, fmt.Errorf("I got an error %v", dur)
}
func testRandomError(d flow.Data) (flow.Data, error) {
	r := rand.Intn(10)
	if r > 5 {
		return nil, errors.New("I failed on purpose")
	}
	return d, nil
}
